package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Civilian
// Camp: Civilians
// New role must have this class as an ancestor.

class Civilian {
	def static order = 0;		// All other roles must set larger value than this.
	def static strings = JinroLocalization.getBundle("roles/Civilian.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	def static isInvisible() {
		return false;
	}
	def getPriority() {
		return 0;
	}
	// Update priority
	// Returns under 0: target -> it
	// Returns over 0 : it -> target
	def getUpdatePriority(target) {
		return 0;
	}
	def actor;
	def Civilian(actor = null) {
		this.actor = actor;
	}
	def getRoleName() {
		return strings.getString("civilian");
	}
	def roleIs(role) {
		return getRoleName() == role.getRoleName();
	}
	def reset() {
		target = null;
		defended = false;
		divined = false;
		itemTarget = null;
	}
	// For the first victim
	def autoCommand(actors) {
		switch(JinroGame.time) {
			case 0:
				def lives = actors.findAll { it.name != name && it.isLive() };
				receiveMessage(strings.getString("command.vote"), lives[Math.floor(Math.random() * lives.size()) as int]);
				break;
			case 1:
				if(JinroGame.day >= 1 && actor.is(JinroGame.getProperty("deathnoteholder"))) {
					if(Math.random() < 0.5) {
						def lives = actors.findAll { it.name != name && it.isLive() };
						receiveMessage(strings.getString("command.deathnote"), lives[Math.floor(Math.random() * lives.size()) as int]);
					}
				}
				break;
		}
	}
	def receiveMessage(type, target) {
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				if(type != strings.getString("command.vote")) {
					return;
				}
				if(this.target) {
					return;
				}
				if(target?.name != name && target?.isLive()) {
					this.target = target;
					notice(strings.getString("notice.vote.target", target.name));
				}
				break;
			case 1:
				// The death note holder only.
				if(actor.is(JinroGame.getProperty("deathnoteholder"))) {
					if(type == strings.getString("command.deathnote")) {
						if(!JinroGame.getProperty("deathnotetarget") && target.name != name && target.isLive()) {
							JinroGame.setProperty("deathnotetarget", target);
							notice(strings.getString("notice.deathnote.target", target.name));
						}
					}
				}
				// If the actor has an unused power item.
				if(powerItem && !itemTarget) {
					if((!roleIs(new Fortuneteller()) && !roleIs(new Wolffortune()) || this.target) && powerItem == strings.getString("poweritem.fortuneteller")) {
						if(type == strings.getString("command.divine") && target) {
							if(target.name != name && target.isLive()) {
								itemTarget = target;
								notice(strings.getString("notice.divine.target", target.name));
								target.divined = true;
								if(target.isWolf()) {
									notice(strings.getString("notice.divine.result", target.name, strings.getString("notice.divine.result.black")));
								} else {
									notice(strings.getString("notice.divine.result", target.name, strings.getString("notice.divine.result.white")));
								}
							}
						}
					}
					if((!roleIs(new Hunter()) || this.target) && powerItem == strings.getString("poweritem.hunter")) {
						if(type == strings.getString("command.defend") && target) {
							if(target.name != name && target.isLive()) {
								itemTarget = target;
								notice(strings.getString("notice.defend.target", target.name));
								target.defended = true;
							}
						}
					}
				}
				break;
		}
	}
	// Calls when start argument/night time.
	def startPhase() {
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				// The death note holder only
				if(actor.is(JinroGame.getProperty("deathnoteholder"))) {
					notice(strings.getString("notice.deathnote"));
				}
				noticePowerItemCommand();
		}
	}
	def noticePowerItemCommand() {
		switch(powerItem) {
			case strings.getString("poweritem.fortuneteller"):
				notice(strings.getString("notice.poweritem.divine"));
				break;
			case strings.getString("poweritem.medium"):
				if(!roleIs(new Medium())) {
					if(JinroGame.getProperty("history")[-1].isWolf()) {
						notice(strings.getString("notice.medium.result", JinroGame.getProperty("history")[-1].name, strings.getString("notice.medium.result.black")));
					} else {
						notice(strings.getString("notice.medium.result", JinroGame.getProperty("history")[-1].name, strings.getString("notice.medium.result.white")));
					}
				}
				break;
			case strings.getString("poweritem.hunter"):
				notice(strings.getString("notice.poweritem.defend"));
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = [];
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.vote") + "@" + it.name;
					}
				}
				break;
			case 1:
				// The death note holder only
				if(actor.is(JinroGame.getProperty("deathnoteholder"))) {
					if(!JinroGame.getProperty("deathnotetarget")) {
						actors.findAll{ it.name != name && it.isLive() }.each {
							commands << strings.getString("command.deathnote") + "@" + it.name;
						}
					}
				}
				// If the actor has an unused power item.
				if(powerItem && !itemTarget) {
					if((!roleIs(new Fortuneteller()) && !roleIs(new Wolffortune()) || target) && powerItem == strings.getString("poweritem.fortuneteller")) {
						actors.findAll{ it.name != name && it.isLive() }.each {
							commands << strings.getString("command.divine") + "@" + it.name;
						}
					}
					if((!roleIs(new Hunter()) || target) && powerItem == strings.getString("poweritem.hunter")) {
						actors.findAll{ it.name != name && it.isLive() }.each {
							commands << strings.getString("command.defend") + "@" + it.name;
						}
					}
				}
				break;
		}
		return commands;
	}
	// Checks if the game can go to next phase.
	def checkStep() {
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				if(target) {
					return true;
				}
				break;
			case 1:
				return true;
		}
		return false;
	}
	// Goes to next phase.
	def update() {
		if(countdown > 0) {
			countdown--;
			if(countdown == 0) {
				killed(nextstate);
			}
		}
	}
	// Checks if the actor is living.
	def isLive() {
		return state.priority > 50;
	}
	// When the actor is killed.
	def killed(state) {
		this.state = state;
		if(state == State.live) {
			noticeKilled(strings.getString("notice.revived", name));
		} else {
			JinroGame.addHistory(this);
			if(state == State.deathnote) {
				noticeKilled(strings.getString("notice.killed.deathnote." + (Math.floor(Math.random() * 4) as int), name));
			} else if(state != State.executed) {
				noticeKilled(strings.getString("notice.killed.normal", name));
			}
		}
	}
	def noticeKilled(message) {
		JinroGame.invokeMethod("noticeKilled", message);
	}
	// If it returns true, the role count as a wolf.
	def isWolf() {
		return false;
	}
	// If it returns true, the role count as a civilian.
	def isCivilian() {
		return true;
	}
	// Notifies the actor of the role.
	def noticeRole() {
		notice(strings.getString("notice.role", JinroGame.getAlias(getRoleName())));
	}
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	def judge(result) {
		return result == 0;
	}
	// Notifies the actor of the message.
	def notice(message) {
		noticelog << message;
		if(skypeid) {
			JinroSkype.notice(skypeid, message);
		}
		JinroGame.addLog(strings.getString("notice.log", name, message));
	}
	// Copy the latest notified message into clipboard.
	def copynoticelog() {
		if(noticelog) {
			JinroInterface.copyClipboard(noticelog[-1]);
		}
	}
	def propertyMissing(String name) {
		return actor.getAt(name);
	}
	def propertyMissing(String name, value) {
		actor.putAt(name, value);
	}
}
