package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Fortuneteller
// Camp: Civilians

class Fortuneteller extends Civilian {
	def static order = 2;
	def static strings = Civilian.strings.attachBundle("roles/Fortuneteller.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 1;
	}
	def getPriority() {
		return 50;
	}
	Fortuneteller(actor = null) {
		super(actor);
	}
	def getRoleName() {
		return strings.getString("fortuneteller");
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				notice(strings.getString("notice.divine"));
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 1:
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.divine") + "@" + it.name;
					}
				}
				break;
		}
		return commands;
	}
	def autoCommand(actors) {
		super.autoCommand(actors);
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				def lives = actors.findAll { it.name != name && it.isLive() };
				receiveMessage(strings.getString("command.divine"), lives[Math.floor(Math.random() * lives.size()) as int]);
				break;
		}
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(type != strings.getString("command.divine")) {
					return;
				}
				if(this.target) {
					return;
				}
				if(target?.name != name && target?.isLive()) {
					this.target = target;
					notice(strings.getString("notice.divine.target", target.name));
					divine();
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				return true;
			case 1:
				if(target) {
					return true;
				}
				break;
		}
		return false;
	}
	def divine() {
		target.divined = true;
		if(target.isWolf()) {
			notice(strings.getString("notice.divine.result", target.name, strings.getString("notice.divine.result.black")));
		} else {
			notice(strings.getString("notice.divine.result", target.name, strings.getString("notice.divine.result.white")));
		}
	}
}
