package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Spy
// Camp: Wolves

class Spy extends Civilian {
	def static order = 9;
	def static strings = Civilian.strings.attachBundle("roles/Spy.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	def escaped = false;
	Spy(actor = null) {
		super(actor);
	}
	def getPriority() {
		return 80;
	}
	def getRoleName() {
		return strings.getString("spy");
	}
	// Notifies the actor of the role.
	def noticeRole() {
		super.noticeRole();
		def actors = JinroGame.getActors();
		actors.findAll{ it.isWolf() }.each {
			notice(strings.getString("notice.wolf", it.name, JinroGame.getAlias((new Wolf()).getRoleName())));
		}
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day < 1) {
					return;
				}
				notice(strings.getString("notice.complete"));
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 1:
				if(JinroGame.day < 1) {
					break;
				}
				if(escaped) {
					break;
				}
				commands << strings.getString("command.complete") + "@";
				break;
		}
		return commands;
	}
	def autoCommand(actors) {
		super.autoCommand(actors);
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day >= 1 && Math.random() < 0.5) {
					receiveMessage(strings.getString("command.complete"), null);
				}
				break;
		}
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 1:
				if(type != strings.getString("command.complete")) {
					return;
				}
				if(JinroGame.day < 1) {
					return;
				}
				escaped = true;
				break;
		}
	}
	// Goes to next phase.
	def update() {
		if(escaped) {
			state = State.escaped;
			escaped = false;
			nextstate = State.live;
			countdown = 0;
			noticeKilled(strings.getString("notice.killed.escaped", name));
		}
		super.update();
	}
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	def judge(result) {
		return result == 1 && state == State.escaped;
	}
}
