package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Wizard
// Camp: Civilians

class Wizard extends Civilian {
	def static order = 14;
	def static strings = Civilian.strings.attachBundle("roles/Wizard.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	def static isInvisible() {
		return false;
	}
	def getPriority() {
		return 30;
	}
	def Wizard(actor = null) {
		super(actor);
	}
	def getRoleName() {
		return strings.getString("wizard");
	}
	def autoCommand(actors) {
		super.autoCommand(actors);
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day >= 2) {
					def deads = actors.findAll { it.name != name && !it.isLive() };
					receiveMessage(strings.getString("command.revive"), deads[Math.floor(Math.random() * deads.size()) as int]);
				}
				break;
		}
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day < 2 || type != strings.getString("command.revive") || this.target) {
					return;
				}
				if(target && target.name != name && !target.isLive() && target.state != State.escaped) {
					this.target = target;
					notice(strings.getString("notice.revive.target", target.name));
				}
				break;
		}
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day >= 2) {
					notice(strings.getString("notice.revive"));
				}
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day >= 2 && !target) {
					actors.findAll{ !it.isLive() && actor.state != State.escaped }.each {
						commands << strings.getString("command.revive") + "@" + it.name;
					}
				}
				break;
		}
		return commands;
	}
	// Checks if the game can go to next phase.
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				return true;
			case 1:
				if(JinroGame.day < 2 || target) {
					return true;
				}
				break;
		}
		return false;
	}
	// Goes to next phase.
	def update() {
		super.update();
		if(JinroGame.day >= 3 && JinroGame.time == 0) {
			if(!target.isLive()) {
				if(Math.random() < (target.is(JinroGame.getProperty("firstvictim")) ? 0.6 : 0.3))
				{
					target.nextstate = State.live;
					target.countdown = 1;
					target.update();
				}
			}
		}
	}
}
